#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#include "view.h"
#include "debug.h"
#include "browse.h"

#include "errcat.h"

/* =========================================================================
   Name: identifier_cat
   Description: Append the identifier to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes: This is the crcid - It is unique for each *type* of error. This
          is used to identify the decoding info from the *undocumented*
	  database.
*/
char *identifier_cat(char *cpstr, int maxchars, struct errline *el)
{
  char idstr[32];
  int colwidth = 9;
  int i, j;

  /* Print out the number with formatting. Insert spaces just to be sure. */
  sprintf(idstr, "%08X            ", el->el_crcid);

  /* Terminate at max length */
  idstr[colwidth] = 0;

  /* Copy to target string */
  i = strlen(cpstr);
  j = 0;

  while( ( i < maxchars ) && ( idstr[j] != 0 ) )
    cpstr[i++] = idstr[j++];

  /* If the previous while loop never fired, then we will simply terminate
     at the previous termination point. */
  cpstr[i] = 0;

  return(cpstr);
}

/* =========================================================================
   Name: timestamp_cat
   Description: Append the SHORTENED time to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes: MM/DD/YY HH:MM:SS
*/
char *timestamp_cat(char *cpstr, int maxchars, struct errline *el)
{
  char timestr[32];
  struct tm *ltime;
  time_t timestamp;
  int colwidth = 18;
  int i, j;

  timestamp = el->el_timestamp;
  ltime = localtime(&timestamp);

  //strftime(timestr, 32, "%m/%d/%C %H:%M:%S", ltime);
  strftime(timestr, 32, "%H:%M:%S %D        ", ltime);
  
  /* Terminate at max length */
  timestr[colwidth] = 0;

  /* Copy to target string */
  i = strlen(cpstr);
  j = 0;
  while( ( i < maxchars ) && ( timestr[j] != 0 ) )
    cpstr[i++] = timestr[j++];

  /* If the previous while loop never fired, then we will simply terminate
     at the previous termination point. */
  cpstr[i] = 0;

  return(cpstr);
}

/* =========================================================================
   Name: longtimestamp_cat
   Description: Append (a regular) timestamp to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes: Standard asctime() timestamp
*/
char *longtimestamp_cat(char *cpstr, int maxchars, struct errline *el)
{
  char timestr[32];
  struct tm *ltime;
  time_t timestamp;
  int colwidth = 25;
  int i, j;

  timestamp = el->el_timestamp;
  ltime = localtime(&timestamp);

  sprintf(timestr, "%s", asctime(ltime));

  /* WHY: Because asctime() puts some ugliness in the string that horks up
     curses. The fix is to hard chomp() the end of the line to fix it. */
  timestr[24] = ' ';
  timestr[25] = 0;

  /* Terminate at max length */
  timestr[colwidth] = 0;

  /* Copy to target string */
  i = strlen(cpstr);
  j = 0;
  while( ( i < maxchars ) && ( timestr[j] != 0 ) )
    cpstr[i++] = timestr[j++];

  /* If the previous while loop never fired, then we will simply terminate
     at the previous termination point. */
  cpstr[i] = 0;

  return(cpstr);
}

/* =========================================================================
   Name: label_cat
   Description: Append the error label to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes:
*/
char *label_cat(char *cpstr, int maxchars, struct errline *el)
{
  char labelstr[64];
  int colwidth = 20;
  int i, j;

  /* Print out the number with formatting. Insert spaces just to be sure. */
  sprintf(labelstr, "%s                        ", el->el_label);

  /* Terminate at max length */
  labelstr[colwidth] = 0;

  /* Copy to target string */
  i = strlen(cpstr);
  j = 0;
  while( ( i < maxchars ) && ( labelstr[j] != 0 ) )
    cpstr[i++] = labelstr[j++];

  /* If the previous while loop never fired, then we will simply terminate
     at the previous termination point. */
  cpstr[i] = 0;

  return(cpstr);
}

/* =========================================================================
   Name: resource_cat
   Description: Append the resource to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes:
*/
char *resource_cat(char *cpstr, int maxchars, struct errline *el)
{
  char rsrcstr[64];
  int colwidth = 15;
  int i, j;

  /* Print out the number with formatting. Insert spaces just to be sure. */
  sprintf(rsrcstr, "%s                        ", el->el_resource);

  /* Terminate at max length */
  rsrcstr[colwidth] = 0;

  /* Copy to target string */
  i = strlen(cpstr);
  j = 0;
  while( ( i < maxchars ) && ( rsrcstr[j] != 0 ) )
    cpstr[i++] = rsrcstr[j++];

  /* If the previous while loop never fired, then we will simply terminate
     at the previous termination point. */
  cpstr[i] = 0;

  return(cpstr);
}

/* =========================================================================
   Name: class_cat
   Description: Append the error class to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes:
*/
char *class_cat(char *cpstr, int maxchars, struct errline *el)
{
  char classstr[16];
  int colwidth = 2;
  int i, j;

  /* Print out the number with formatting. Insert spaces just to be sure. */
  sprintf(classstr, "%s   ", el->el_class);

  /* Terminate at max length */
  classstr[colwidth] = 0;

  /* Copy to target string */
  i = strlen(cpstr);
  j = 0;
  while( ( i < maxchars ) && ( classstr[j] != 0 ) )
    cpstr[i++] = classstr[j++];

  /* If the previous while loop never fired, then we will simply terminate
     at the previous termination point. */
  cpstr[i] = 0;

  return(cpstr);
}

/* =========================================================================
   Name: rclassrtype_cat
   Description: Append either rclass or rtype to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes: if rtype is "NONE" then rclass will be used
*/
char *rclassrtype_cat(char *cpstr, int maxchars, struct errline *el)
{
  char rstr[32];
  int colwidth = 16;
  int i, j;

  /* Print out the number with formatting. Insert spaces just to be sure. */
  if ( (el->el_rclass[0] == 'N') &&
       (el->el_rclass[1] == 'O') &&
       (el->el_rclass[2] == 'N') &&
       (el->el_rclass[3] == 'E') )
  {
    sprintf(rstr, "%s                ", el->el_rtype);

  }
  else
    sprintf(rstr, "%s                ", el->el_rclass);

  /* Terminate at max length */
  rstr[colwidth] = 0;

  /* Copy to target string */
  i = strlen(cpstr);
  j = 0;
  while( ( i < maxchars ) && ( rstr[j] != 0 ) )
    cpstr[i++] = rstr[j++];

  /* If the previous while loop never fired, then we will simply terminate
     at the previous termination point. */
  cpstr[i] = 0;

  return(cpstr);
}

/* =========================================================================
   Name: sequence_cat
   Description: Append the sequence number to input string
   Input: char *           : String to append to
          int              : Maximum chars in string to utilize
          struct errline * : The structure of error info for this error 
   Returns: The string, modified
   Side Effects: If sufficient space exists, the string will be modified
   Notes:
*/
char *sequence_cat(char *cpstr, int maxchars, struct errline *el)
{
  char seqstr[32];
  int colwidth = 14;
  int i, j;

  /* If it is in the actual system log we print a sequence number, otherwise
     we print a * to denote that the error has been deleted */
  if(el->bIsInLog)
  {
    /* Print out the number with formatting. Insert spaces just to be sure. */
    sprintf(seqstr, "%d               ", el->el_sequence);
  }
  else
  {
    /* If the error is not in the log, then it no longer has a sequence # */
    strcpy(seqstr, "*                   ");
  }

  /* Terminate at max length */
  seqstr[colwidth] = 0;

  /* Copy to target string */
  i = strlen(cpstr);
  j = 0;
  while( ( i < maxchars ) && ( seqstr[j] != 0 ) )
    cpstr[i++] = seqstr[j++];

  /* If the previous while loop never fired, then we will simply terminate
     at the previous termination point. */
  cpstr[i] = 0;

  return(cpstr);
}
