#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>
#include <ctype.h>

#include "view.h"
#include "debug.h"
#include "browse.h"
#include "search.h"
#include "errcat.h"
#include "indexit.h"

#include "errdata.h"

/*** Globals ***/
extern struct errres *error_data;


/* ========================================================================= */
/* top_seq_is_old()
    Returns a boolean (int) of is out of sync or is not. The choice of "sync"
    is intentional as it means more than "old". If all errors have been
    cleared from the system, then this will return true. This is NOT
    technically old, but it means that update_list_data() needs to fix the
    list.
*/
int top_seq_is_old(unsigned int saved_topseq)
{
  errlog_handle_t eh;
  unsigned int app_top_seq;
  unsigned int sys_top_seq;
  errlog_entry_t eresult;
  int retval;
  
  retval = 0; /* Set our return value now to a safe value */

  app_top_seq = saved_topseq;

  /* Open the error log */
  if(0 != errlog_open(NULL, O_RDONLY, LE_MAGIC, &eh))
  {
    /* We failed to open the log. return(), don't exit(). This thread should
       be robust and not die early, just keep trying. */
    return(0);
  }

  /* Grab the top of the error log */
  if(0 == errlog_find_next(eh, &eresult))
  {
    /* Got one */
    sys_top_seq = eresult.el_sequence;
  }
  else
  {
    /* Discuss: If we did not get an error, then most likely they have all
       been cleared. This means that things have changed. */
    retval = 1;
  }
      
  errlog_close(eh);

  if(sys_top_seq != app_top_seq)
    retval = 1;


  return(retval);
}

/* ========================================================================= */
/* The data structure is locked then unlocked while in this function. */
int update_list_data(unsigned int *saved_topseq)
{
  errlog_handle_t eh;
  unsigned int sys_top_seq;  /* This will be used to reset top value on exit */
  unsigned int last_top_seq; /* This is the last seq number we grabbed       */
  unsigned int list_top_seq; /* This is what is the top of the list at start */
  errlog_entry_t eresult;
  int egot;

  struct errline *errlast = NULL;
  struct errline *errthis = NULL;
  struct errline *new_list_top = NULL;
  struct errline *new_list_bottom = NULL;


  TDebugOut("Top sequence ID %d is old.\n", *saved_topseq);

  list_top_seq = *saved_topseq;

  /* initialize last_top_seq to saved_topseq to get us through the
     first test. This will be a safe non-valid value. */
  last_top_seq = *saved_topseq;
  
  /* Initialize sys_top_sequence to a known non-top value */
  sys_top_seq = *saved_topseq;

  /* Initialize linked list info */
  egot = 0;
  new_list_top = NULL;
  new_list_bottom = NULL;


  LockData(); /* ===== DATA LOCKED ===== */

  /* The list will be changed. Determine if the screen should be re-focused
     onto the new list. We determine here (in the thread code) so that we
     do not have to save various pointers and make potentially more ugly
     determinations later. refocus is *always* set when updating the list,
     it is simple, and clean.

     The rules are thus:

     - If cursor is at the top of the list, then it is ok to auto-scroll
       under the user.

     - If the cursor is *anywhere* but the top of the list, then do not
       auto scroll. The use may be in the middle of a search, browsing, or
       microseconds before hitting the view key.
  */
  if(error_data->cursor_el == error_data->el)
  {
    /* We can auto-scroll under the user */
    error_data->refocus = 1;
  }
  else
  {
    /* Do NOT auto-scroll under the user */
    error_data->refocus = 0;
  }

  /* Open the error log */
  if(0 != errlog_open(NULL, O_RDONLY, LE_MAGIC, &eh))
  {
    /* We failed to open the log. return(), don't exit(). This thread should
       be robust and not die early, just keep trying. */
    return(1);
  }

  /* Step through the NEW errors */
  if( (0 == errlog_find_next(eh, &eresult)) &&
      (*saved_topseq <= last_top_seq) )
  {
    /* If we foune a new top sequence, then reset the top */
    if(sys_top_seq < eresult.el_sequence)
      sys_top_seq = eresult.el_sequence;

    if(eresult.el_sequence > list_top_seq)
    {
      /* Allocate memory for struct */
      if(NULL == (errthis = (struct errline *)malloc(sizeof(struct errline))))
      {
	/* If unable to allocate memory - shutdown the thread! */
	UnlockData(); /* ===== DATA UNLOCKED ===== */
	errlog_close(eh);
	TDebugOut("ERROR: Unable to allocate memory. Thread quitting.\n");
	pthread_exit(NULL);
      }

      errthis->bIsInLog = 1; /* We know this, we just got it! */

      errthis->el_sequence = eresult.el_sequence;
      errthis->el_timestamp = eresult.el_timestamp;
      errthis->el_errdiag = eresult.el_errdiag;
      errthis->el_crcid = eresult.el_crcid;
      strcpy(errthis->el_label, eresult.el_label);
      strcpy(errthis->el_rclass, eresult.el_rclass);
      strcpy(errthis->el_rtype, eresult.el_rtype);
      strcpy(errthis->el_class, eresult.el_class);
      strcpy(errthis->el_resource, eresult.el_resource);
      /* At least one system was seeing this as a problem */
      errthis->el_resource[LE_RESOURCE_MAX - 1] = 0;
      
      /** Linked list work **/

      /* Forward links */

      errthis->next = NULL;

      /* This is 2nd or later error */
      if(NULL != errlast)
	errlast->next = errthis;

      /* Backward links */
      errthis->last = NULL;

      if(NULL != errlast)
	errthis->last = errlast;

      /* Save a reference to this for the next time */
      errlast = errthis;

      /* Save top of list */
      if(NULL == new_list_top)
      {
	/* Insert at top */
	new_list_top = errthis;
      }

      /* Save bottom of list */
      new_list_bottom = errthis;

      egot++;
      
      TDebugOut(" List insert (%d).\n", eresult.el_sequence);
    }

    last_top_seq = eresult.el_sequence;
  }

  errthis = error_data->el;
  while(errthis)
  {
    /* Walk remainder of list looking for changes */
    if(0 != errlog_find_sequence(eh, errthis->el_sequence, &eresult))
      errthis->bIsInLog = 0;

    errthis = errthis->next;
  }

  /* We are done with the system error log */
  errlog_close(eh);


  /* Update top sequence value(s) */
  *saved_topseq = sys_top_seq;
  error_data->el_topseq = sys_top_seq;

  /* Join lists */
  error_data->el->last = new_list_bottom;
  new_list_bottom->next = error_data->el;
  error_data->el = new_list_top;

  /* Update total got */
  error_data->got += egot;

  /* Set the dirty flag */
  error_data->dirty = 1;

  UnlockData(); /* ===== DATA UNLOCKED ===== */
  
  return(0);
}



/* Notes on the sleep option:

 - The code here uses nanosleep() but I do not think it is appropriate to
   check for updates more than once a second. The nanosleep() may be used
   elsewhere - updater threads or something like that. The expectation is
   that nanosleep() will be replaced with sleep();

 - If the options are to present the user with the capability of less than
   second granularity, then the number must be manipulated. For example: 1.5
   seconds must be divided into 1 second and 500000000 nanoseconds.

 - 1,000,000,000 nanoseconds is one second. This value overflows the tv_nsec
   member (even though tv_nsec is a long).
*/ 

#define ONE_WHOLE_SECOND 1000000000
#define ONE_HALF_SECOND  500000000
#define ONE_TENTH_SECOND 100000000

/* ========================================================================= */
/* This will be the background thread that checks for updates */
void *UpdaterThread(void *args)
{
  struct timespec rqtp;
  unsigned int saved_topseq;

  TDebugOut("Background thread started.\n");

  rqtp.tv_sec = 1;
  rqtp.tv_nsec = ONE_HALF_SECOND;

  /*** GRAB MUTEX LOCK ***/
  LockData();
  /*   Grab the top sequence number here so that the data structure is only
       locked once (for checks). This code (update_list_data()) is the only
       code that will update it so it is ok to keep a local copy. */
  saved_topseq = error_data->el_topseq;
  UnlockData();
  /*** RELEASE MUTEX LOCK ***/
  
  while(1)
  {
    if(top_seq_is_old(saved_topseq))
      update_list_data(&saved_topseq);

    nanosleep(&rqtp, NULL);
  }

  return(NULL);
}

/* ========================================================================= */
struct errres *InitializeErrorData(int count)
{
  struct errres *er;       /* errbr (local) data structure                   */

  /* Look for nutty input */
  if(count <= 0)
    return(NULL);

  /* This is the memory for the parent structure */
  if(NULL == (er = (struct errres *)malloc(sizeof(struct errres))))
    return(NULL);

  /* Initialize the parent structure */
  er->soft_max = count;
  er->hard_max = count + 10240; /* STUB: This number is made up */
  er->got = 0;
  er->el_topseq = 0;
  er->el = NULL;     /* This is the important one */
  er->eol = NULL;
  er->dirty = 0;
  er->refocus = 1; /* No need to initialize this value. It is ALWAYS reset
		      when dirty is set, and ONLY used after dirty has been
		      set. */

  /* Initialize screen pointers */
  er->top_screen_el = NULL;
  er->cursor_el = NULL;
  er->winx = 0;

  return(er);
}

/* ========================================================================= */
int BuildErrorList(void)
{
  int i;
  errlog_handle_t eh;      /* "File" handle to error log   - IBM data struct */
  errlog_entry_t eresult;  /*                                                */

  /* Linked list member and place-holder */
  struct errline *errlast;
  struct errline *errthis = NULL;



  /* GOAT: We need to insert a filter here. I believe that you place the filter
           on the errlog_open() call, not the find next call. At any rate, this
           has to happen to filter on CLASS.
  */


  /* Open - if error then return null. Should free memory but if NULL is
     returned we are not long for this world anyways. */
  if(0 != errlog_open(NULL, O_RDONLY, LE_MAGIC, &eh))
  {
    return(0);
  }

  error_data->got = 0;

  i = 0; /* i is no longer an index, just a counter */
  while(i < error_data->soft_max)
  {
    if(0 == errlog_find_next(eh, &eresult))
    {
      if(NULL == (errthis = (struct errline *)malloc(sizeof(struct errline))))
      {
	/* Don't clean up, give up. If we are this sideways, don't try to
	   fix, just exit as gracefully as possible. */
	return(0);
      }

      errthis->bIsInLog = 1;

      errthis->el_sequence = eresult.el_sequence;
      errthis->el_timestamp = eresult.el_timestamp;
      errthis->el_errdiag = eresult.el_errdiag;
      errthis->el_crcid = eresult.el_crcid;
      strcpy(errthis->el_label, eresult.el_label);
      strcpy(errthis->el_rclass, eresult.el_rclass);
      strcpy(errthis->el_rtype, eresult.el_rtype);
      strcpy(errthis->el_class, eresult.el_class);
      strcpy(errthis->el_resource, eresult.el_resource);
      /* At least one system was seeing this as a problem */
      errthis->el_resource[LE_RESOURCE_MAX - 1] = 0;
      
      /** Linked list work **/

      /* Forward links */

      errthis->next = NULL;

      /* This is 2nd or later error */
      if(NULL != errlast)
	errlast->next = errthis;


      /* Backward links */
      errthis->last = NULL;

      if(NULL != errlast)
	errthis->last = errlast;

      /* Save a reference to this for the next time */
      errlast = errthis;

      /* Save top of list */
      if(NULL == error_data->el)
      {
	/* Insert at top */
	error_data->el = errthis;
	error_data->el_topseq = errthis->el_sequence;
      }

      /* Save bottom of list */
      error_data->eol = errthis;

      error_data->got++;
    }
    else
    {
      errlog_close(eh);
      error_data->top_screen_el = error_data->el;
      error_data->cursor_el = error_data->el;
      error_data->winx = 0;
      return(i);
    }

    i++;
  }

  error_data->top_screen_el = error_data->el;
  error_data->cursor_el = error_data->el;
  error_data->winx = 0;
  return(i);
}







