#ifndef LISTDATA_H
#define LISTDATA_H

#include <sys/errlog.h>
#include <pthread.h>
#include <curses.h>


/* ========================================================================= */
/* Structure Name: errline
   Description: Linked list member
   Notes: Largely a copy of the IBM datatype, but with in-log boolean and
          linked list enabling data types.
*/
struct errline
{
  /** Unique to this implementation **/
  int bIsInLog;              /* If the data is still in the log */

  /** Copied from the AIX data type ***/
  unsigned int el_sequence;
  unsigned int el_timestamp;
  unsigned int el_errdiag;
  unsigned int el_crcid;
  char el_label[LE_LABEL_MAX];
  char el_rclass[LE_RCLASS_MAX];
  char el_rtype[LE_RTYPE_MAX];
  char el_class[LE_CLASS_MAX];
  char el_resource[LE_RESOURCE_MAX];

  /** Linked list components **/
  struct errline *last; /* last is newer than current. On newest/latest last
			   will be NULL. Use last to walk from eol (end of
			   list) to el (error list). */
  struct errline *next; /* next is older than current. On the oldest error
			   on the list (with the lowest sequence) this will
			   be NULL. Use next to walk from el to eol. */
};

/* Here is the deal:
   This is depricated. Not used any more. I keep it for the commentary. Once
   the "deleted" flag (isInLog) was added, we could never use the IBM supplied
   data type (plus it is now a linked list).

   Dissusion that follows in this comment block is historical.

   I use the typedef of single_error so that I can later substitute for the
   actual AIX data type. Currently I am using my own data type because it is
   considerably smaller. This does not matter much on modern hardware but it
   is proportionally significant. My struct uses the same member names so that
   one can be swapped for another later. There is currently no plan to swap.

   THIS MUST BE DEPRICATED. WE WILL BE ADDING A "DELETED" FLAG TO THE STRUCT
   THAT WILL TELL THE USER VISUALLY THAT THE ITEM HAS BEEN DELETED (POSSIBLY
   BY ANOTHER USER).
*/
/* typedef struct errline single_error; */



/* ========================================================================= */
/* Structure Name: errres
   Description: Primary data access for the entire application
   Notes: This is the primary data access handle. It is used to place all
          data in one pointer (for clean paramater lists) as well as provide
	  a single point to mutex on (to avoid locking conditions). Additional
	  documentation is included in the structure. 
*/
struct errres
{
  pthread_mutex_t data_lock;

  int got;       /* How many items we got - Used primarily for page and list
		    movement actions to insure that we go to the end of the
		    list of items. I believe this may be supplanted by the
		    "offset" based code in indexed.c/h.  */
  int soft_max;  /* Maximum number of items to retrieve */
  int hard_max;  /* Do something drastic if this is exceeded (to be defined) */

  int dirty;     /* Means this was updated. Set by thread. Unset when display
		    code refreshes display. */

  int refocus;   /* When we find that the list was updated (is "dirty") then
		    we need to determine is we need to re-position the cursor
		    and screen. This will tell us to do that or not. This was
		    required because it was essential to know when we re-draw
		    the screen if the cursor *was* at the top of the list or
		    not. There were a few methods of determining this - all
		    not as simple as this. */

  unsigned int el_topseq; /* This is the top (highest) sequence number in the
			     linked list. */

  struct errline *el;    /* Head of list */
  struct errline *eol;   /* End of list */

  /* Window references
   +-------------------------------------------------------------------------+
   |<top_screen_el>                                                          |
   |                                                                         |
   |                                                                         |
   |<winx = 3><cursor_el> Cursor is here!                                    |
   |                                                                         |
   |                                                                         |
   |                                                                         |
   |                                                                         |
   |                                                                         |
   |                                                                         |
   +-------------------------------------------------------------------------+
   */

  struct errline *top_screen_el;
  struct errline *cursor_el;
  int winx; /* 0 based offset value */

  /* Window geometery. 1 based values */
  /* Note that the X axis is lines and Y is columns. This is a bit different
     than how curses or a cartesian graph treats these values. The "error"
     snuck into an early version of the code and has stayed ever since. Such
     is life. */
  int maxx;
  int maxy;
  WINDOW *window;

};







/* ========================================================================= */
/* Fairly simple... the idea is that we can pass a paramater to lock a
   range. This paramater can be a bit-field that can mask off the data
   required. This may be too complex and prone to deadlocks - but if we
   do that then we can possibly use conditional locks... Either way, this
   is a method to do this.
*/
void InitLock(void); /* Called once */

void LockData(void);

void UnlockData(void);

#endif
