#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <pthread.h>
#include <curses.h>

#include "browse.h"
#include "debug.h"
#include "options.h"
#include "view.h"
#include "version.h"

#include "errdata.h"

/* Major modes - Right now there is only one. The plan was to have all the
   modes selectable from here. As it stands the *minor* modes are selectable
   as options from the major. I am leaving this as there *may* be another
   major mode in the future such as search, a selective browse, or the
   ability to browse other items.
*/
#define MODE_QUIT   0
#define MODE_BROWSE 1


/*** Prototypes ***/
int init_main_window(void);
void usage(void);

/*** Globals ***/
struct errres *error_data;


/* ========================================================================= */
int main(int argc, char *argv[])
{
  struct options *o;
  int brv;
  pthread_t bstid;   /* Background Scan Thread ID */

  /*** Parse input to main() ***/
  if(NULL == (o = ParseOptions(argc, argv)))
  {
    fprintf(stderr, "ERROR: Unable to parse options.\n");
    return(1);
  }

  /*** Process immediate user requests / errors ***/
  if(o->bError)
  {
    printf("ERROR: Input not understood.\n");
    return(1);
  }

  if(o->bSaveOpts)
  {
    return(SaveOptions(o));
  }

  if(o->bVersion)
  {
    printf("errbr - Version: %s\n", VERSION_STRING);
  }

  if(o->bHelp)
  {
    usage();
  }

  if( (o->bVersion) || (o->bHelp) )
    return(0);
    
#ifdef DEBUG_CL_INPUT
  if(o)
  {
    if(o->filterclass != 0)
      printf("Filtering for \"%c\"\n", o->filterclass);

    printf("MAX_ERRORS_TO_PULL = %d\n", o->maxerrpull);

    printf("CURSOR_TYPE = ");
    switch (o->cursortype)
    {
    case CURSOR_GT:
      printf("GT\n");
      break;
    case CURSOR_REV:
    default:
      printf("REV\n");
      break;
    }

    return(0);
  }




#endif

  /* Check to insure that we have the errpt binary */
  if(!GotErrptBinary())
  {
    /* It is too shamefull to admit that I rely upon another binary. */
    fprintf(stderr, "ERROR: Unable to find required support files.\n");
    return(1);
  }


  DebugOut("\n-----------------------------------\nStarting App.\n");


  /* Initialize the top struct before thread. This is done because this struct
     contains the thread mutex. It must be checkable BEFORE we start a thread.
  */
  if(NULL == (error_data = InitializeErrorData(o->maxerrpull)))
  {
    printf("ERROR: Unable to initialize error list.\n");
    return(1);
  }

  /* Retrieve a list of errors */
  if(BuildErrorList() == 0)
  {
    printf("ERROR: Unable to build the initial error list.\n");
    return(EBR_INTERNAL_ERROR);
  }

  /* Exit here if we did not get any errors. */
  if(0 == error_data->got)
  {
    printf("ERROR: Error log contains no events.\n");
    return(EBR_EMPTY_ERROR_LOG);
  }

  InitLock(); /* Setup the lock on the data structure (do not set it) */

  DebugOut("Got %d errors.\nTop sequence is %d.\n",
	   error_data->got,
	   error_data->el_topseq);

  DebugOut("Launching thread.\n");

  /*** Start thread to watch log ***/ 
  if(pthread_create(&bstid, NULL, UpdaterThread, NULL))
  {
    fprintf(stderr, "ERROR: Unable to create background thread.\n");
    return(1);
  }
  /* Thread is off and running */


  /*** Begin curses mode operations ***/
  if(init_main_window() != 0)
  {
    fprintf(stderr, "Unable to initialize curses.");
    return(1);
  }

  if(80 > error_data->maxy)
  {
    endwin();
    fprintf(stderr, "Problem with screen width (too narrow).\n");
    return(1);
  }

  /* Begin the primary event loop of the application: Browse(); */
  brv = Browse(o);
  /* We have left the primary event loop. The show is over. */

  /* Shutdown curses */
  endwin();


  switch(brv)
  {
  case EBR_INTERNAL_ERROR:
    fprintf(stderr, "pull_error_list() => NULL\n");
    break;
  case EBR_EMPTY_ERROR_LOG:
    fprintf(stderr, "No errors found in error log.\n");
    break;
  case EBR_OK:
  default:
    printf("\n"); /* AIX curses does not clear the final line. */
    fflush(stdout);
    break;
  }

  return(0);
}



/* ========================================================================= */
int init_main_window(void)
{
  if(NULL == error_data)
    return(1);

  /* Safe defaults */
  error_data->maxx = 0;
  error_data->maxy = 0;
  error_data->winx = 0;
  
  /* Initialize the window */
  error_data->window = initscr();

  /* Save the default program mode (because we do not call initscr() ) */
  def_prog_mode();

  /* Make the cursor invisible */
#ifndef CURSOR_VISIBLE
  curs_set(0);
#endif

  scrollok(error_data->window, 1);

  /* Do not wait for CR on input */
  cbreak();

  /* We want to get the Ctrl-c, not get SIGINT */
  raw();

  /* Do not echo input */
  noecho();

  /* This is used to NOT wait for control character sequences. Without this
     we are dropping into the meta sequence handler on some key presses that
     should NOT be handled there. */
  notimeout(error_data->window, TRUE);

#ifdef USE_CURSES_KEYS
  /* Enable keypad keys */
  keypad(error_data->window, TRUE);
#endif

  /* Find the window size. Why pass by value and not reference? Because it
     is a macro and not an actual function.

     NOTE: X and Y values are OPPOSITE of the definition in the API!
  */
  getmaxyx(error_data->window, error_data->maxx, error_data->maxy);

  /* Clear the screen */
  wclear(error_data->window);

  refresh();

  return(0);
}


void usage(void)
{
  printf("Usage: errbr [options]\n");
  printf(" Options:\n");
  printf("   -h, --help            Show this help\n");
  printf("   -v, --version         Show version string\n");
  printf("   -b <type>             Change browse type. Valid types are:\n");
  printf("                           t - Show \"long\" time format\n");
  printf("                           i - Show ID with shorter time format (default)\n");
  printf("   -H                    Show headers in browse mode (default)\n");
  printf("   -I                    No headers in browse mode\n");
#ifdef UNDEFINED
  printf("   -c <class>              Filter for class (H, S, O, or U)\n");
#endif
  printf("   -C                    Use carrat cursor method\n");
  printf("   -R                    Use reverse cursor method (default)\n");
  printf("   --set-defaults        Save current options to ~/.errbr\n");
  printf(" Using the app:\n");
  printf("  Browse using emacs or vi keys.\n");
  printf("  View by v or space over selected broswse item.\n");
  printf("  Return to browse mode by q or space.\n");
  printf("  Exit the application from browse more using q\n");
  printf("\n");
  printf("Browse Key Help:                                                               \n");
  printf("_Action_________________vi_______________emacs__________Other__________________\n");
  printf(" move up                k                Ctrl-p         up arrow               \n");
  printf(" move down              j                Ctrl-n         down arrow             \n");
  printf(" page up                Ctrl-b           Ctrl-v                                \n");
  printf(" page down              Ctrl-f           Esc-v                                 \n");
  printf(" to top                 1                Esc->                                 \n");
  printf(" to bottom              G                Esc-<                                 \n");
  printf(" refresh screen         Ctrl-l                                                 \n");
  printf(" quit/exit                               Ctrl-x Ctrl-c  q                      \n");
  printf(" view selected item                                     v, Space, or Enter     \n");
  printf(" search                 /                Ctrl-s         s, or S                \n");
  printf(" find next              n                               N                      \n");
  printf("View Key Help:                                                                 \n");
  printf("_Action_________________vi_______________emacs__________Other__________________\n");
  printf(" move up                k                Ctrl-p         up arrow               \n");
  printf(" move down              j                Ctrl-n         down arrow             \n");
  printf(" to top                 1                Esc->                                 \n");
  printf(" to bottom              G                Esc-<                                 \n");
  printf(" return to browse mode                                  q, Space, or Enter     \n");
}
