#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <fcntl.h>
#include <sys/errlog.h>
#include <ctype.h>

#include "options.h"
#include "debug.h"
#include "indexit.h"
#include "browse.h"
#include "search.h"


/* This is the maximum length of the search string. Check for this to prevent
   buffer overflow issues. */
#define SRCH_STR_MAX 128

/*** Globals ***/
extern struct errres *error_data;

#ifdef UNDEFINED
/* These were prototyped in search.h - unnessarily as they are not public
   functions. Plus, they were prototyped WRONG. Plus, they used the wrong
   naming convention. Plus, search.h was not included here.... This was all
   cleaned up, but this UNDEFINED block was kept in the event that ordering
   of these functions was required, and prototypes would some day be required
   again. (Changes are expected in this file, post version 2.0.) */

/* ========================================================================= */
int set_found_item(struct options *o,
		   struct errline *item);

/* ========================================================================= */
int find_item(struct options *o,
	      char *needle,
	      int offset);

/* ========================================================================= */
int find_first_item(struct options *o,
		    char *needle);

char *get_search_str(struct options *o,
		     char *needle);


#endif


/* ========================================================================= */
int set_found_item(struct options *o,
		   struct errline *item)
{
  int hdroffset; /* Offset for the header */
  int fioffset;  /* Found Item Offset (from cursor to item) */

  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    hdroffset = 1;
    break;
  case BO_NOSHOHEADR:
  default:
    hdroffset = 0;
    break;
  }

  /* Set found item to top of screen (if possible) */
  /* Set pointer / highlight to found item */

  fioffset = ListOffsetValue(error_data->cursor_el, item, LOD_FORWARD);

  /* Is the found item off the screen? */
  if(error_data->winx + fioffset + 1 > error_data->maxx)
  {
    /* We must scroll */
    error_data->top_screen_el = ListOffsetMember(item,
						 (error_data->maxx - 1) - hdroffset, 
						 LOD_REVERSE);
    error_data->cursor_el = item;
    error_data->winx = error_data->maxx - 1;
  }
  else
  {
    /* Item is on the current screen */
    error_data->cursor_el = item;
    /* Header offset is already accounted for */
    error_data->winx = error_data->winx + fioffset; 
  }

  /* Redraw screen */
  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    refresh_hdrbr_page(o);
    break;
  case BO_NOSHOHEADR:
  default:
    refresh_browse_page(o);
    break;
  }

  return(0);
}

/* ========================================================================= */
int find_item(struct options *o,
	      char *needle,
	      int offset)
{
  struct tm *ltime;
  time_t timestamp;
  char haystack[128];
  char timestr[32];
  struct errline *startpt; /* Slightly misleading name. It will be used to
			      walk the list (so it is not always the "start").
			   */

  /* The point of offset is to say if we want to find the item we are on
     or not. An offset of 0 says start searching where we are. An offset of
     1 says to start searching on the *next* item. So the first search is
     with an offset of 0 (begin searching from here) and the second +
     searches are about looking at everything thereafter. */
  if(offset == 0)
    startpt = error_data->cursor_el;
  else
    startpt = error_data->cursor_el->next; 

  if(NULL == needle)
    return(1);

  if(0 == needle[0])
    return(1);

  while(startpt)
  {
    if(NULL != strstr(startpt->el_label, needle))
      return(set_found_item(o, startpt));
    
    timestamp = startpt->el_timestamp;
    ltime = localtime(&timestamp);

    /* "Standard" time string */
    sprintf(haystack, " %s", asctime(ltime));
    if(NULL != strstr(haystack, needle))
      return(set_found_item(o, startpt));

    /* The "shortened" time string that I (tend to) use */
    strftime(haystack, 32, "%H:%M:%S %D", ltime);
    strftime(timestr, 32, "%H:%M:%S %D", ltime);
    if(NULL != strstr(haystack, needle))
      return(set_found_item(o, startpt));

    /* Smells like a cheat to me!
       This is just the "full" browse line. It contains checks that we
       have already done. */
    sprintf(haystack, "%-8d %-19s %1s %-13s %-15s %s",
	    startpt->el_sequence,
	    startpt->el_label,
	    startpt->el_class,
	    startpt->el_rclass,
	    startpt->el_resource,
	    timestr);
    if(NULL != strstr(haystack, needle))
      return(set_found_item(o, startpt));

    startpt = startpt->next;
  }

  return(1);
}

/* ========================================================================= */
int find_first_item(struct options *o,
		    char *needle)
{
  return(find_item(o, needle, 0));
}

/* ========================================================================= */
int find_next_item(struct options *o,
				   char *needle)
{
  return(find_item(o, needle, 1));
}


/* ========================================================================= */
char *get_search_str(struct options *o,
		     char *needle)
{
  char srchline[error_data->maxy + 1];
  int sc;
  int i;
  int x;

  if(NULL == needle)
  {
    if(NULL == (needle = (char *)malloc(SRCH_STR_MAX)))
      return(NULL);

    needle[0] = 0;
  }

  strcpy(srchline, "Search: ");
  strcat(srchline, needle);

  /* i is used temporarily to clear out the search line */
  i = strlen(needle) + 8;
  x = i;
  while(i < (error_data->maxy - 1))
    srchline[i++] = ' ';
  srchline[i] = 0;

  mvaddstr(error_data->maxx - 1, 0, srchline);

  sc = 0;
  /* i now has position of cursor in actual string */
  i = strlen(needle);

  /* Make the cursor invisible */
#ifndef CURSOR_VISIBLE
  curs_set(1);
  move(error_data->maxx - 1, x);
#endif

  DebugOut("Search...\n");

  while ((sc != 10) && (i < SRCH_STR_MAX))
  {
    sc = getch();

    DebugOut("%1c %3d :", sc, sc);

    /* if(isprint(sc)) - Removed this in favor of isalnum() */
    if(isalnum(sc))
    {
      move(error_data->maxx - 1, x);
      addch(sc);
      needle[i] = sc;
      i++;
      x++;
    }

    /* 8=BackSpace, 127=DEL */
    if( ((8 == sc) || (127 == sc)) && (i > 0) )
    {
      x--;
      i--;
      move(error_data->maxx - 1, x);
      addch(' ');
      /* Adding the char moves us, so we need to move (back) again. */
      move(error_data->maxx - 1, x);
      needle[i] = 0;
    }

    DebugOut(" i=%d x=%d\n", i, x);
  }

  /* Make the cursor invisible */
#ifndef CURSOR_VISIBLE
  curs_set(0);
#endif

  needle[i] = 0; /* Terminate */

  DebugOut("Searching for \"%s\"\n", needle);

  /* Re-draw the bottom line */
  /* Why we use refresh_browse_page() instead of:
     add_browse_line(o, sd, sd->maxx - 1, &er->el[sd->topindex + sd->maxx - 1]);
     1) Because it does not cost much
     2) It can clear any other artifacts that *might* be on the screen
     3) Because the message line might have the cursor / highlight on it.
  */
  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    refresh_hdrbr_page(o);
    break;
  case BO_NOSHOHEADR:
  default:
    refresh_browse_page(o);
    break;
  }

  return(needle);
}

/* ========================================================================= */
/* Note: This *should* use var-args (like printf) */
int DisplayMessage(struct options *o,
		    const char *message)
{
  char msgline[error_data->maxy + 1];
  char fmtline[32];
  int i;

  /* Create the format line as we do not know the screen size */
  sprintf(fmtline, "%%-%ds", error_data->maxy - 2);
  sprintf(msgline, fmtline, message);

  /* Make the rest of the line into spaces */
  i = strlen(msgline) - 1;
  while(i < error_data->maxy - 1)
    msgline[i++] = ' ';
  /* Terminate */
  msgline[i] = 0;

  /* Display the message */
  mvaddstr(error_data->maxx - 1, 0, msgline);
  
  /* Await *any* user input */
  getch();

  /* Redraw the page */
  switch( o->bShowBrHeader )
  {
  case BO_SHOWHEADER:
    refresh_hdrbr_page(o);
    break;
  case BO_NOSHOHEADR:
  default:
    refresh_browse_page(o);
    break;
  }

  return(0);
}

/* ========================================================================= */
/* The key difference (between this and mv_next()) is that this will always
   ask for the search string. mv_next() will only ask for the search string
   if it is not set.
*/
char *MoveFirst(struct options *o,
		char *needle)
{
  needle = get_search_str(o, needle);

  if(0 != find_first_item(o, needle))
  {
    /* Item not found */
    DisplayMessage(o, "Search item not found.");
  }

 return(needle);
}

/* ========================================================================= */
char *MoveNext(struct options *o,
	       char *needle)
{
  if(NULL == needle)
    needle = get_search_str(o, needle);

  if(0 != find_next_item(o, needle))
  {
    /* Item not found */
    DisplayMessage(o, "Search item not found.");
  }

  return(needle);
}


























