#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>

#include "view.h"
#include "debug.h"
#include "options.h"

/* We can freely hard-set the width because we always know that errpt (our
   current source of data) never goes wider than 80. (Although errpt may
   be using the screen size.) */
#define RPT_LINE_SZ  88
#define RPT_LINE_CNT 100

/* Commentary on view_data:

 - The initial design created a two-dimensional array of characters, then
   relied a bit too loosely on a mix of hard coded offsets (RPT_LINE_SZ) and
   window size to determine offsets. This worked fine for a "standard" sized
   window, but was entirely too problematic for odd-sized windows.

 - The two dimensional array was a single mallocation. The new method of
   an array of pointers uses multiple memory allocations. The new uses a bit
   less memory but takes a bit longer. Both of these issues are largely
   irrelevant for the nature of the task at hand as run on even 10 year-old
   CPUs (that I am developing on).
*/

/* ========================================================================= */
struct view_data
{
  unsigned int sequence;
  char filename[64];
  int total_data_lines; /* Like it says. A total line count. (NOT 0-based) */
  int top_line;         /* 0-based index of top line of the display        */
  /* char errpt_data[RPT_LINE_CNT + 1][RPT_LINE_SZ + 1]; */
  char *errpt_lines[RPT_LINE_CNT];
  WINDOW *w;
};


/* ========================================================================= */
struct view_data *init_view_data(WINDOW *w, unsigned int sequence)
{
  struct view_data *v;
  int i;

  if(NULL == (v = (struct view_data *)malloc(sizeof(struct view_data))))
  {
    return(NULL);
  }

  /* Set the sequence */
  v->sequence = sequence;

  /* Pre-set the filename */
  sprintf(v->filename, "/tmp/errbr.%lu", (unsigned long)getpid());

  /* Initialize the line count & position */
  v->total_data_lines = 0;
  v->top_line = 0;

  /* Initialize the array */
  i = 0;
  while(i < RPT_LINE_CNT)
    v->errpt_lines[i++] = NULL;

  /* Save the WINDOW pointer */
  v->w = w;

  return(v);
}

/* ========================================================================= */
void delete_view_data(struct view_data *v)
{
  int i;

  if(NULL == v)
    return;

  /* If any array position has memory allocated, free it */
  i = 0;
  while(i < RPT_LINE_CNT)
  {
    if(NULL == v->errpt_lines[i])
      free(v->errpt_lines[i]);

    i++;
  }

  free(v);
}


/* ========================================================================= */
/* This is code stolen from dump_page() that starts at a location other than
   the first line. */
int view_refresh_page(struct view_data *vd)
{
  int sline; /* Screen line */
  int dline; /* Data line */
  int maxx, maxy;

  getmaxyx(vd->w, maxx, maxy);

  erase();

  /* Dump starting at current top line */
  dline = vd->top_line;
  sline = 0;
  while ( (sline < maxx) && (dline < vd->total_data_lines) )
  {
    if(NULL != vd->errpt_lines[dline])
    {
      if(strlen(vd->errpt_lines[dline]) > maxy)
	vd->errpt_lines[dline][maxy - 1] = 0;

      mvaddstr(sline, 0, vd->errpt_lines[dline]);
    }
    else
    {
      /* Some (unlikely) internal error caused a NULL in the middle of
	 our expected data. Handle it semi-gracefully, and move on. */
      mvaddstr(sline, 0, "~");
    }

    sline++;
    dline++;
  }


  while(sline < maxx)
  {
    mvaddstr(sline, 0, "~");
    sline++;
  }

  move(maxx - 1, maxy - 1);

  return(dline);
}

/* ========================================================================= */
int dump_page(struct view_data *vd)
{
  int line;
  int maxx, maxy;

  getmaxyx(vd->w, maxx, maxy);

  /* Dump first page */
  line = 0;
  while ( (line < maxx) && (line < vd->total_data_lines) )
  {
    if(NULL != vd->errpt_lines[line])
    {
      if(strlen(vd->errpt_lines[line]) >= maxy)
	vd->errpt_lines[line][maxy - 1] = 0;

      mvaddstr(line, 0, vd->errpt_lines[line]);
    }
    else
    {
      /* Some (unlikely) internal error caused a NULL in the middle of
	 our expected data. Handle it semi-gracefully, and move on. */
      mvaddstr(line, 0, "~");
    }

    line++;
  }


  while(line < maxx)
  {
    mvaddstr(line, 0, "~");
    line++;
  }

  move(maxx - 1, maxy - 1);

  return(line);
}

/* ========================================================================= */
void chomp(char *cp)
{
  char *eol;

  if(NULL != (eol = strchr(cp, '\n')))
    *eol = 0;

  /* We cut the line because we know errpt does not display wider than 80 */
  /* cp[79] = 0;    YOINK! This is ugly, wrong to assume, unsafe, and not
                           part of the "stated" definition of the funciton. */
}

/* ========================================================================= */
int load_errpt_dump(struct view_data *v)
{
  char cmdstr[64];
  FILE *edf;
  char line[RPT_LINE_SZ + 1];

  if(NULL == v)
    return(1);

  /* Build command string and run it */
  sprintf(cmdstr,
	  "errpt -al %lu > %s",
	  (unsigned long)v->sequence,
	  v->filename);

  if(0 != system(cmdstr))
    return(1);

  /* Open file we just created from the system call. */
  if(NULL == (edf = fopen(v->filename, "r")))
    return(1);

  v->total_data_lines = 0;
  while( (fgets(line, RPT_LINE_SZ, edf)) && (v->total_data_lines < RPT_LINE_CNT) )
  {
    chomp(line);

    /* * /
    DebugOut("[%d]%s|\n",
	     v->total_data_lines,
	     line);
    / * */

    if(NULL != (v->errpt_lines[v->total_data_lines] = (char *)malloc(strlen(line) + 1)))
    {
      /* strncpy not required. We only read RPT_LINE_SZ max characters anyways */
      strcpy(v->errpt_lines[v->total_data_lines], line);
    }
    /* else { leave the member as null } */
    /* The liklihood of failing on an allocation of this size is exceptionally
       low. I am  not going to write an explicit exception for it. If you fail
       on an allocation like this then something *serious* is up and the user
       will be quite aware that something is going on. So we will just deal
       with NULL lines if/when they exist. If the user cannot allocate less
       than 100 bytes, they will be unlikely to get this far anyways as the
       system call to errpt would have failed. */

    v->total_data_lines++;
  }

  fclose(edf);
  unlink(v->filename);
  /*
  DebugOut("Read %d lines.\n", vd->total_data_lines);
  */
  return(0);
}


/* ========================================================================= */
int view_scroll_up(struct view_data *vd)
{
  int maxx, maxy;

  getmaxyx(vd->w, maxx, maxy);

  if(vd->top_line > 0)
  {
    scrl(-1); 
    vd->top_line--;

    if(NULL != vd->errpt_lines[vd->top_line])
    {
      if(strlen(vd->errpt_lines[vd->top_line]) >= maxy)
	vd->errpt_lines[vd->top_line][maxy - 1] = 0;

      mvaddstr(0, 0, vd->errpt_lines[vd->top_line]);
    }
    else
    {
      /* Some (unlikely) internal error caused a NULL in the middle of
	 our expected data. Handle it semi-gracefully, and move on. */
      mvaddstr(0, 0, "~");
    }

  }

  move(maxx - 1, maxy - 1);

  return(0);
}

/* ========================================================================= */
int view_scroll_down(struct view_data *vd)
{
  int maxx, maxy;

  getmaxyx(vd->w, maxx, maxy);

  if(vd->top_line + maxx < vd->total_data_lines)
  {
    scrl(1);
    vd->top_line++;

    if(NULL != vd->errpt_lines[(maxx - 1) + vd->top_line])
    {
      if(strlen(vd->errpt_lines[(maxx - 1) + vd->top_line]) >= maxy)
	vd->errpt_lines[(maxx - 1) + vd->top_line][maxy - 1] = 0;

      mvaddstr(maxx - 1, 0, vd->errpt_lines[(maxx - 1) + vd->top_line]);
    }
    else
    {
      /* Some (unlikely) internal error caused a NULL in the middle of
	 our expected data. Handle it semi-gracefully, and move on. */
      mvaddstr(maxx - 1, 0, "~");
    }
  }

  move(maxx - 1, maxy - 1);

  return(0);
}

/* ========================================================================= */
int view_page_down(struct view_data *vd)
{
  int maxx, maxy;

  getmaxyx(vd->w, maxx, maxy);

  while(--maxx)
  {
    view_scroll_down(vd);
  }

  return(0);
}


/* ========================================================================= */
int view_page_up(struct view_data *vd)
{
  int maxx, maxy;

  getmaxyx(vd->w, maxx, maxy);

  while(--maxx)
  {
    view_scroll_up(vd);
  }

  return(0);
}


/* ========================================================================= */
int view_goto_top(struct view_data *vd)
{
  if(vd->top_line > 0)
  {
    vd->top_line = 0;

    view_refresh_page(vd);
  }
  return(0);
}

/* ========================================================================= */
int view_goto_bottom(struct view_data *vd)
{
  int maxx, maxy;

  getmaxyx(vd->w, maxx, maxy);

  if(maxx < vd->total_data_lines)
  {
    vd->top_line = vd->total_data_lines - maxx;

    view_refresh_page(vd);
  }
  return(0);
}

/* ========================================================================= */
/* Used to support emacs Esc key sequences                                   */
int view_meta_second_key(struct view_data *vd)
{
  int c;
  
  c = getch(); 

  switch(c)
  {
  case '>':
    view_goto_bottom(vd);
    break;

  case '<':
    view_goto_top(vd);
    break;

  case 'v':
    view_page_down(vd);
    break;

  }

  return(1);
}

/* ========================================================================= */
int handle_view(struct view_data *vd)
{
  int c;
  int maxx, junk;
  int in_mode = 1;

  getmaxyx(vd->w, maxx, junk);

  move(maxx, 0);

  while(in_mode)
  {
    c = getch(); 

    switch(c)
    {
    case 'q':
    case 'Q':
    case 32: /* Space bar gets you here, and it takes you back. */
    case 10: /* RET gets you here, and it takes you back */
    /* This is triggered on down arrow on at least one system. */
    /* case 27: - Esc seems like a reasonable way to get back */
      in_mode = 0;
      break;

    case KEY_SR:
    case KEY_UP:
    case 65:  /* KEY_UP arrow keys    */
    case 'k': /* vi binding    */
    case 16:  /* emacs binding (^P) */
      /* up */
      view_scroll_up(vd);
      break;

    case KEY_BEG:
    case KEY_HOME:
    case '0': /* vi binding (Esc-:-0) - top */
    case '1': /* vi binding (1G) - top */
      view_goto_top(vd);
      break;

    case KEY_END:
    case 'G': /* vi binding - bottom */
      view_goto_bottom(vd);
      break;

    case KEY_NPAGE:
    case 6: /* vi binding - Ctrl+F */
      view_page_down(vd);
      break;

    case KEY_PPAGE:
    case 22:/* emacs binding - Ctrl+v */
    case 2: /* vi binding - Ctrl+B */
      view_page_up(vd);
      break;

    case KEY_SF:
    case KEY_DOWN:
    case 66:  /* KEY_DOWN arrow keys    */
    case 'j': /* vi binding    */
    case 14:  /* emacs binding (^N) */
      /* down */
      view_scroll_down(vd);
      break;

    case 27:  /* Esc (meta) key  [emacs] */
      view_meta_second_key(vd);
      break;

    case 12: /* vi binding - Ctrl+l */
      view_refresh_page(vd);
      break;

    default:
      /* When a key press is not understood, then ignore it */
      break;
    }

    refresh();
  }

  return(0);
}



/* ========================================================================= */
int ViewItem(unsigned int item, WINDOW *w)
{
  struct view_data *vd;

  /* Initialize structure */
  if(NULL == (vd = init_view_data(w, item)))
    return(1);

  if(load_errpt_dump(vd))
    return(1);

  erase();

  dump_page(vd);

  refresh();

  handle_view(vd);

  delete_view_data(vd);

  return(0);
}



/* ========================================================================= */
int GotErrptBinary(void)
{
  char errptbin[] = "/usr/bin/errpt"; /* really unnessary */
  struct stat statbuf;

  /* Just look for the file. Don't sweat stuff like SUID, execute, etc... */
  /* We will assume that it is SUID and executable */
  if(0 != stat(errptbin, &statbuf))
    return(0);


  return(1);

}
